//
// Displays a cube with different colored sides. The user can rotate around the
//    x, y, and z axis by pressing the "x" and "y" and "z" keys, or hit "r" to reset.
//  The x rotation is done first, followed by y, then z


#include "Angel.h"
#include "Cube.h"
#include <iostream>
using std::cout;


// Viewing transformation parameters
GLfloat theta = 0.0;
GLfloat phi = 0.0;
GLfloat alpha = 0.0;

Cube myCube;  // generates the vertices, colors, normals of a cube.

// Cmamera projection transformation parameters
GLfloat  fovy = 45.0;  // Field-of-view in Y direction angle (in degrees)
GLfloat  aspect;       // Viewport aspect ratio
GLfloat  zNear = 0.5, zFar = 100.0;

GLuint  projection; // projection matrix uniform shader variable location

// Camera location and orientation parameters
vec4 eye = vec4( 0.0 ,0.0, 2.0 ,1.0); // camera location
vec4  u( 1.0, 0.0, 0.0, 0.0);
vec4  v( 0.0, 1.0, 0.0, 0.0);
vec4  n( 0.0, 0.0, 1.0, 0.0 );  // camera looks along -n

GLuint  model_view;  // model-view matrix uniform shader variable location
GLuint program;


//---------------------------------------------------------------------------- printControls

void printControls()
{
    cout << "\n\n************  Controls **************" << "\n";
    cout << "q or Q ............ quit" << "\n";
    cout << "x ................. rotate around x" << "\n";
    cout << "y ................. rotate around y" << "\n";
    cout << "z ................. rotate around z" << "\n";
    cout << "r ................. reset" << "\n";
}

//---------------------------------------------------------------------------- init
// OpenGL initialization
void
init()
{
    program = InitShader( "vertex.glsl", "fragment.glsl" );
    glUseProgram(program );

    myCube.createBuffers(program);

    // Uniform variables
    model_view = glGetUniformLocation( program, "model_view" );
    projection = glGetUniformLocation( program, "projection" );


    glEnable( GL_DEPTH_TEST );
    glClearColor( 0.5, 0.5, 0.5, 1.0 );
    printControls();
}




//----------------------------------------------------------- display

void
display( void )
{
    // clear the color and depth buffers
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // set camera location and orientation
    vec4 t = vec4(0.0, 0.0, 0.0, 1.0); // needed for homogeneous coordinates
    mat4 c = mat4(u, v, n, t);         // matrix for orienting the camera
    mat4 mv =  c * Translate( -eye );  // matrix for moving and orienting the camera
    glUniformMatrix4fv( model_view, 1, GL_TRUE, mv );
   // cout << " mv = \n" << mv << "\n";

    // set camera projection parameters: field of view, aspect ratio, near and far planes
    mat4  p  = Perspective( fovy, aspect, zNear, zFar );
    glUniformMatrix4fv( projection, 1, GL_TRUE, p );  // set projection uniform variable in shader
    //cout << " p = \n" << p << "\n";

    mv = mv*RotateZ(alpha);   // rotate about z axis
    mv = mv*RotateY(theta);   // rotate about y axis
    mv = mv*RotateX(phi);     // rotate about x axis
    glUniformMatrix4fv( model_view, 1, GL_TRUE, mv ); // set modelview uniform variable in shader

    myCube.draw();    // draw the cube

    glBindVertexArray( 0 );
    glutSwapBuffers();
}


//---------------------------------------------------------------------------- keyboard

void
keyboard( unsigned char key, int x, int y )
{
    float inc = 1.0;
    switch( key )
    {
    case 033: // Escape Key
    case 'q':
    case 'Q':
        exit( EXIT_SUCCESS );
        break;

    case 'y':   // rotate around y
        theta += 5;
        break;
    case 'x':     // rotate around x  (this is always applied second)
        phi += 5;
        break;
    case 'z':     // rotate around z  (this is always applied second)
        alpha += 5;
        break;
    case 'r':     // reset
        phi = 0;
        theta = 0;
        alpha = 0;
        break;
    }
    glutPostRedisplay();
}


//---------------------------------------------------------------------------- reshape

void
reshape( int width, int height )
{
    glViewport( 0, 0, width, height );
    aspect = GLfloat(width)/height;
}


//---------------------------------------------------------------------------- main

int
main( int argc, char **argv )
{
    glutInit( &argc, argv );  // initialize window
    glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
    glutInitWindowSize( 512, 512 );
    glutInitContextVersion( 3, 3 );   // set version of opengl are we using
    glutInitContextProfile( GLUT_CORE_PROFILE );  // set opengl context profile, see
                            // http://www.opengl.org/wiki/Core_And_Compatibility_in_Contexts
    glutCreateWindow( "Cube Test" );

    glewInit();

    init();  // initialize various parameters

    // call back functions (i.e. event handlers)
    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
    glutReshapeFunc( reshape );

    glutMainLoop();  // start display loop
    return 0;
}
